@@ -3,8 +3,8 @@
 
 from ClickSQL.clickhouse.ClickHouse import ClickHouseTableNode
 from ClickSQL.clickhouse.ClickHouseExt import ClickHouseTableNodeExt
-from ClickSQL.nodes.factor_node import BaseSingleFactorTableNode
+from ClickSQL.nodes.base import BaseSingleFactorTableNode
 
-__version__ = '0.1.7.6'
+__version__ = '0.1.7.10'
 __author__ = 'sn0wfree'
 __all__ = ['ClickHouseTableNode', 'ClickHouseTableNodeExt', 'BaseSingleFactorTableNode']
