@@ -10,6 +10,7 @@ import pandas as pd
 from scipy.stats import t
 
 from ClickSQL.utils import boost_up, cached_property
+import warnings
 
 # fig = plt.figure()
 # sns.set_palette("GnBu_d")
@@ -71,15 +72,28 @@ class DefaultModel(object):
         return cls.cal_ar(*args, **kwargs)
 
     @staticmethod
-    def cal_ar():
+    def cal_ar(*args, **kwargs):
         raise ValueError('cal_ar have not been defined!')
 
 
 class EventStudyUtils(object):
     @staticmethod
     def split_event(return_df: pd.DataFrame, event_happen_day: str, stock, date='Date', factors=['Mkt_RF'],
-                    event_info: tuple = (250, 20, 10, 1), ):
-        variables = [date, stock] + factors
+                    event_info: tuple = (250, 20, 10, 1), detect=True):
+        if isinstance(stock, str):
+            variables = [date, stock] + factors
+        elif isinstance(stock, list):
+            variables = [date] + stock + factors
+        else:
+            raise TypeError('stock got wrong type! only accept str or list')
+        if detect:
+            if event_info[-2] < 0:
+                if event_info[-1] > abs(event_info[-2]) + 1:
+                    pass
+                else:
+                    warnings.warn('estimation period covered event date! will shift estimation period!')
+                    event_info = (event_info[0], event_info[1], event_info[2], abs(event_info[-2]) + 1)
+
         event_set = event_tuple(*event_info)
         post_event_window = event_set.post_event_window
         after_event_window = event_set.event_window - post_event_window
@@ -87,7 +101,8 @@ class EventStudyUtils(object):
             data = return_df[variables]
         except KeyError as e:
             raise EventDataKeyError(e)
-        event_index_loc = int(data[data[date] == str(event_happen_day)].index.values)
+        event_index_loc = int(
+            (pd.to_datetime(data[date]) - pd.to_datetime(event_happen_day)).abs().sort_values().index[0])
         estimation_df = data.loc[event_index_loc - (
                 event_set.before_event_window + event_set.post_event_window + event_set.gap): event_index_loc - (
                 event_set.post_event_window + event_set.gap), variables]
@@ -98,6 +113,20 @@ class EventStudyUtils(object):
 
         return estimation_df, event_df
 
+    @staticmethod
+    def cal_ar_multi_no_split_event(estimation_df, event_df, stock: str, factors=['Mkt_RF'],
+                                    ar_only=True, model=DefaultModel):
+        if issubclass(model, DefaultModel):
+            ar_series = model.real_run(estimation_df, event_df, stock, factors=factors)
+        else:
+            raise TypeError(f'model got wrong definition: {model.__name__}! should be a DefaultModel-liked class')
+        if ar_only:
+            # output_cols = [f'{stock}_ar']
+            return pd.DataFrame(ar_series)
+        else:
+            event_df[f'{stock}_ar'] = ar_series
+            return event_df
+
     @classmethod
     def cal_ar_single(cls, return_df: pd.DataFrame, event_happen_day: str, stock: str,
                       date='Date', factors=['Mkt_RF'], event_info: tuple = (250, 20, 10, 1), ar_only=True,
@@ -120,20 +149,41 @@ class EventStudyUtils(object):
         # expected returns for each firm in the estimation window
         # event_df = data.loc[event_index - event_set.post_event_window: event_index + after_event_window,
         #            variables].reset_index(drop=True)
-        if issubclass(model, DefaultModel):
-            ar_series = model.real_run(estimation_df, event_df, stock, factors=factors)
-        else:
-            raise TypeError(f'model got wrong definition: {model.__name__}! should be a DefaultModel-liked class')
+        # if issubclass(model, DefaultModel):
+        #     ar_series = model.real_run(estimation_df, event_df, stock, factors=factors)
+        # else:
+        #     raise TypeError(f'model got wrong definition: {model.__name__}! should be a DefaultModel-liked class')
+        #
+        # # event_df[f'{stock}_er'] = np.dot(event_df[factors].values, beta_Mkt) + alpha
+        # # event_df[f'{stock}_ar'] = event_df.eval(f'{stock} - {stock}_er')
+        # # event_df.index = event_df.index - event_set.post_event_window
+        # if ar_only:
+        #     # output_cols = [f'{stock}_ar']
+        #     return pd.DataFrame(ar_series)
+        # else:
+        #     event_df[f'{stock}_ar'] = ar_series
+        #     return event_df
+        return cls.cal_ar_multi_no_split_event(estimation_df, event_df, model, stock, factors=factors,
+                                               ar_only=ar_only, )
 
-        # event_df[f'{stock}_er'] = np.dot(event_df[factors].values, beta_Mkt) + alpha
-        # event_df[f'{stock}_ar'] = event_df.eval(f'{stock} - {stock}_er')
-        # event_df.index = event_df.index - event_set.post_event_window
-        if ar_only:
-            # output_cols = [f'{stock}_ar']
-            return pd.DataFrame(ar_series)
-        else:
-            event_df[f'{stock}_ar'] = ar_series
-            return event_df
+    @classmethod
+    def group_split_event(cls, return_df: pd.DataFrame, event_dict: dict, date='Date',
+                          factors=['Mkt_RF'],
+                          event_info: tuple = (250, 20, 10, 1)):
+        # tasks_pre = ((stock, event_happen_day) for stock, event_happen_day in event_dict.items())
+        sub_tasks = {}
+        for stock, event_happen_day in event_dict.items():
+            if event_happen_day in sub_tasks.keys():
+                sub_tasks[event_happen_day].append(stock)
+            else:
+                sub_tasks[event_happen_day] = [stock]
+        for event_happen_day, stock_list in sub_tasks.items():
+            group_estimation_df, group_event_df = cls.split_event(return_df, event_happen_day, stock_list, date=date,
+                                                                  event_info=event_info, factors=factors)
+            for stock in stock_list:
+                yield group_estimation_df, group_event_df, stock
+
+        # tasks = ((return_df, event_happen_day, stock) for stock, event_happen_day in tasks_pre)
 
     @classmethod
     def cal_residual(cls, return_df: pd.DataFrame, event_dict: dict, date='Date',
@@ -157,15 +207,18 @@ class EventStudyUtils(object):
         :param boost:
         :return:
         """
-        func = partial(cls.cal_ar_single, date=date, factors=factors, model=model,
-                       event_info=event_info, ar_only=ar_only)
+        # func = partial(cls.cal_ar_single, date=date, factors=factors, model=model,
+        #                event_info=event_info, ar_only=ar_only)
+        func = partial(cls.cal_ar_multi_no_split_event, factors=factors, model=model, ar_only=ar_only)
 
-        tasks = ((return_df, event_happen_day, stock) for stock, event_happen_day in event_dict.items())
+        # tasks = ((return_df, event_happen_day, stock) for stock, event_happen_day in event_dict.items())
+        tasks = cls.group_split_event(return_df, event_dict, date=date, factors=factors, event_info=event_info)
 
         if boost:
             holder = boost_up(func, tasks, star=True)
         else:
-            holder = [func(return_df, event_happen_day, stock=stock) for _, event_happen_day, stock in tasks]
+            holder = [func(group_estimation_df, group_event_df, stock) for group_estimation_df, group_event_df, stock in
+                      tasks]
         return pd.concat(holder, axis=1)
 
     # @staticmethod
@@ -262,24 +315,28 @@ class EventStudy(EventStudyUtils):
         cols = return_df.columns.tolist()
         new_event_dict = {}
         for k, v in event_dict.items():
-
+            if k not in cols:
+                raise ValueError(f'{k} not found')
+            data_series = return_df[k]
             if isinstance(v, str):
-
                 new_k = k + f"_{pd.to_datetime(v).strftime('%Y%m%d')}"
                 new_event_dict[new_k] = v
-                return_df[new_k] = return_df[k]
+                return_df[new_k] = data_series
             elif isinstance(v, (list, tuple)):
                 v = list(set(v))
 
                 if len(v) >= 1:
+                    # cols = return_df.columns.tolist()
                     # new_event_dict[k] = v
-                    for dt in v:
-                        new_k = k + f"_{pd.to_datetime(dt).strftime('%Y%m%d')}"
-                        if k in cols:
-                            return_df[new_k] = return_df[k]
-                        else:
-                            raise ValueError(f'{k} not found')
-                        new_event_dict[new_k] = dt
+                    new_dt_k_dict = {k + f"_{pd.to_datetime(dt).strftime('%Y%m%d')}": dt for dt in v}
+                    # for dt in v:
+                    #     new_k = k + f"_{pd.to_datetime(dt).strftime('%Y%m%d')}"
+                    extra_return_df = pd.DataFrame([data_series.values.ravel() for _ in v],
+                                                   index=list(new_dt_k_dict.keys()), columns=data_series.index).T
+                    return_df = pd.concat([return_df, extra_return_df], axis=1)
+                    # return_df = return_df.reindex(columns=cols +list(new_dt_k_dict.keys()) ,fill_value=)
+
+                    new_event_dict.update(new_dt_k_dict)
                 else:
                     raise ValueError(f'event list is empty for {k}')
             else:
@@ -311,7 +368,7 @@ class EventStudy(EventStudyUtils):
 
         :return:
         """
-        return self.residual.mean(axis=1)
+        return self.residual.mean(axis=1, skipna=True)
 
     @cached_property
     def std_ar(self):
